%include common-macros.specinclude

# subpackages, they all build together
# they like to use a mix of major.minor.release and only major.minor
%define tk_ver 8.5.9
%define tk_minor 8.5
%define tcl_ver 8.5.9
%define tcl_minor 8.5
%define expect_ver 5.43.0
%define expect_minor 5.43

Summary: An embeddable scripting language.
Name: lsb-tcl
Version: %{tcl_ver}
Release: 21.lsb5
Vendor: Linux Foundation
Packager: LSB project team <lsb-discuss@linuxfoundation.org>
License: BSD
Group: Appbat/languages
Buildroot: %{_tmppath}/%{name}-root
AutoReqProv: no
Requires: lsb >= 5.0
BuildRequires: lsb-build-libbat-%{appbatversion}
BuildRequires: lsb-build-cc

Source0: http://easynews.dl.sourceforge.net/sourceforge/tcl/tcl%{tcl_ver}-src.tar.gz
Source1: http://easynews.dl.sourceforge.net/sourceforge/tk/tk%{tk_ver}-src.tar.gz
Source2: http://expect.nist.gov/src/expect-%{expect_ver}.tar.gz
Patch0: tcl%{tcl_ver}.patch
Patch1: tcl8.5.1-expect.patch
Patch2: tk%{tk_ver}.patch
Patch3: expect-%{expect_minor}.patch
Patch4: expect-tcl8.5.patch
Patch5: tcl%{tcl_ver}-tests.patch

%description
LSB conforming version of tcl.  Tcl is a simple scripting language
designed to be embedded in other applications.  This package also includes
tclsh, a simple example of a Tcl application.  Tcl is added to the LSB
Application Battery primarly because the included test suite exercises
many LSB features; it also provides a dependency for the LSB conforming
version of expect.

%package -n lsb-expect
Summary: a tcl extension for driving interactive programs
Version: %{expect_ver}
Vendor: Linux Foundation
Packager: Appbat Team <lsb-appbat@linux-foundation.org>
License: BSD
Group: Appbat/languages
AutoReqProv: no
Requires: lsb >= 5.0
Requires: lsb-tcl >= 8.5

%description -n lsb-expect
LSB conforming version of expect.  Expect is a tcl extension for
automating and testing interactive applications such as telnet, ftp,
passwd, fsck, rlogin, tip, etc.  Expect is added to the LSB Application
Battery primarly to demonstrate package dependecies, as it depends on LSB
conforming tcl.  It is also used in building the LSB Runtime Test suite.

%prep
%setup -q -c -a0 -a1 -a2

cd tcl%{tcl_ver}
%patch0 -p1 -b .lsbcc
%patch1 -p1 -b .tcl-expect
%patch5 -p0 -b .lsb-test
cd ..
cd tk%{tk_ver}
%patch2 -p1 -b .lsbcc
cd ..
cd expect-%{expect_minor}
%patch3 -p1 -b .lsbcc
%patch4 -p1 -b .expect-tcl
cd ..

%build
# common environment settings
export LSBCC_LSBVERSION=%{appbatversion}
%define prefix /opt/lsb
%define app_prefix %{prefix}/appbat
%define bindir %{app_prefix}/bin
%define sharedir %{app_prefix}/share
%define mandir %{app_prefix}/man

%ifarch s390x ppc64 x86_64
%define lib lib64
%else
%define lib lib
%endif
%define libdir %{prefix}/%{lib}
%define includedir %{prefix}/include

# need to be able to align with the LSB/libbat version we're built for
export CC="lsbcc -L/opt/lsb/libbat/%{appbatversion}/%{lib} -I/opt/lsb/libbat/%{appbatversion}/include"
export CXX="lsbc++ -L/opt/lsb/libbat/%{appbatversion}/%{lib} -I/opt/lsb/libbat/%{appbatversion}/include"
%define libdir %{prefix}/%{lib}-%{appbatversion}
export PKG_CONFIG_PATH=%{libdir}/pkgconfig:/opt/lsb/libbat/%{appbatversion}/%{lib}/pkgconfig
export PATH=/opt/lsb/libbat/%{appbatversion}/bin:${PATH}

# so tcl/tk can find each other to build
export LSBCC_SHAREDLIBS=tcl%{tcl_minor}:tk%{tk_minor}

# first tcl (config)
cd tcl%{tcl_ver}/unix
./configure \
    --prefix=%{app_prefix} \
    --libdir=%{app_prefix}/%{lib}

# don't optimize for s390x
%ifarch s390x
sed -i 's|CFLAGS_OPTIMIZE               =|#CFLAGS_OPTIMIZE   =|g' Makefile
%endif

# don't set USE_TERMIOS so we avoid ioctl's
sed -i 's|-DUSE_TERMIOS=1 ||g' Makefile
cd ../..

# now tk (config)
cd tk%{tk_ver}/unix
./configure \
    --prefix=%{app_prefix} \
    --libdir=%{app_prefix}/%{lib} \
    --x-includes=%{includedir}/X11

cd ../..

# build tcl/tk 
make -C tcl%{tcl_ver}/unix
make -C tk%{tk_ver}/unix

# now expect
# bug 2916 - on some systems configure decides it is OK to user stropts.h - use lsbcpp
export CPP=lsbcpp

# and now this breaks with lsbcpp?
export ac_cv_type_pid_t=yes

# configure expect
# need tcl/tk locations
export TCL_ROOT=`pwd`/tcl%{tcl_ver}
export TK_ROOT=`pwd`/tk%{tk_ver}

cd expect-%{expect_minor}

# more lib/lib64 fun'n'games - try to get everything to use one or the other
sed -i 's|@TCL_EXEC_PREFIX@/lib|@TCL_EXEC_PREFIX@/%{lib}|g' Makefile.in

./configure \
    --prefix=%{app_prefix} \
    --libdir=%{app_prefix}/%{lib} \
    --cache-file=/dev/null \
    --with-tcl=$TCL_ROOT/unix \
    --with-tclinclude=$TCL_ROOT \
    --with-tk=$TK_ROOT/unix \
    --with-tkinlcude=$TKROOT

# expect can't use "installed" location for tcl because we just built it: 
# it's not installed there yet. Since we point an odd place, we set rpath 
# so it will find the eventual installed location.

sed -i 's|TCLLIB_INSTALLED = -L%{app_prefix}/%{lib} -ltcl%{tcl_minor}|TCLLIB_INSTALLED = -Wl,-rpath,%{app_prefix}/%{lib} $(TCLLIB)|g' Makefile
sed -i 's|TKLIB_INSTALLED = -L%{app_prefix}/%{lib} -ltk%{tk_minor}|TKLIB_INSTALLED = -Wl,-rpath,%{app_prefix}/%{lib} $(TKLIB)|g' Makefile
# only finds part of what it needs in "generic"
sed -i "s|TCLHDIRDASHI = -I$TCL_ROOT/generic|TCLHDIRDASHI = -I$TCL_ROOT/generic -I$TCL_ROOT/unix|g" Makefile
sed -i 's|{exec_prefix}/lib|{exec_prefix}/%{lib}|g' Makefile

# build expect
# FIXME x86_64 only seems to think it needs to rebuild configure and friends at this point
touch configure config.status Makefile

# gripes about ptrdiff_t difference in headers
make XCFLAGS="-DSTDC_HEADERS=1 -DHAVE_UNISTD_H=1"

%install
rm -rf %{buildroot}

# tcl/expect use themselves to install; need to be able to find libtcl in build area for it to work
export LD_LIBRARY_PATH=$TCL_ROOT/unix

# tcl
make -C tcl%{tcl_ver}/unix INSTALL_ROOT=%{buildroot} install-binaries install-libraries install-tzdata

# tk
make -C tk%{tk_ver}/unix INSTALL_ROOT=%{buildroot} install-binaries install-libraries

# version agnostic tclsh, wish
cd %{buildroot}/%{bindir}
ln -sf tclsh%{tcl_minor} tclsh
ln -sf wish%{tk_minor} wish
cd -

# expect
# does more compiling?
export LSBCC_SHAREDLIBS=tcl%{tcl_minor}:tk%{tk_minor}

make -C expect-%{expect_minor} INSTALL_ROOT=%{buildroot} install-binaries install-libraries

# make READMEs about the patches
for patch in %{PATCH0} %{PATCH1} %{PATCH2} %{PATCH5}; do 
  COUNT=`grep -n '\-\-\-' $patch | head -1 | awk -F\: '{print $1}'`
  if [ "$COUNT" -gt 1 ];then
    COUNT=`expr $COUNT - 1`
    install -d %{buildroot}%{sharedir}/doc
    head -"$COUNT" $patch >> %{buildroot}%{sharedir}/doc/README.LSB.tcl.build.patches 
  fi
done
for patch in %{PATCH3} %{PATCH4}; do
  COUNT=`grep -n '\-\-\-' $patch | head -1 | awk -F\: '{print $1}'`
  if [ "$COUNT" -gt 1 ];then
    COUNT=`expr $COUNT - 1`
    install -d %{buildroot}%{sharedir}/doc
    head -"$COUNT" $patch >> %{buildroot}%{sharedir}/doc/README.LSB.expect.build.patches 
  fi
done

# rpm's strip on some systems complains about perms
chmod 0755 %{buildroot}/%{app_prefix}/%{lib}/lib*.so

# still lib/lib64 inconsistencies, other .tcl files are all in lib
if [ -f %{buildroot}/%{app_prefix}/lib64/tk%{tk_minor}/pkgIndex.tcl ];then
  mv %{buildroot}/%{app_prefix}/lib64/tk%{tk_minor}/pkgIndex.tcl %{buildroot}/%{app_prefix}/lib/tk%{tk_minor}
  rm -fr  %{buildroot}/%{app_prefix}/lib64/tk%{tk_minor}
fi

# trim/install our tests
rm -f tcl%{tcl_ver}/tests/*.lsb-test
rm -f tcl%{tcl_ver}/tests/httpold.test
# make a generic synlink so we don't need to keep changing the FVT/autotest
cd %{buildroot}/%{app_prefix}/lib
ln -sf tcl%{tcl_minor} tcl
cd -
cp -ar tcl%{tcl_ver}/tests %{buildroot}/%{app_prefix}/lib/tcl%{tcl_minor}

%pre

%post

%preun

%postun

%clean
rm -rf %{buildroot}

%files
%defattr(-,root,root)
%{bindir}/tclsh*
%{bindir}/wish*
%{app_prefix}/include/tcl*.h
%{app_prefix}/include/tk*.h
%{app_prefix}/%{lib}/libtcl*
%{app_prefix}/%{lib}/tclConfig.sh
%{app_prefix}/%{lib}/libtk*
%{app_prefix}/%{lib}/tkConfig.sh
%{app_prefix}/lib/tcl*
%{app_prefix}/lib/tk%{tk_minor}
%{sharedir}/doc/README.LSB.tcl.build.patches

%files -n lsb-expect
%defattr(-,root,root)
%{bindir}/expectk
%{bindir}/rlogin-cwd
%{bindir}/mkpasswd
%{bindir}/weather
%{bindir}/xkibitz
%{bindir}/cryptdir
%{bindir}/decryptdir
%{bindir}/passmass
%{bindir}/rftp
%{bindir}/expect
%{bindir}/xpstat
%{bindir}/timed-read
%{bindir}/ftp-rfc
%{bindir}/unbuffer
%{bindir}/timed-run
%{bindir}/multixterm
%{bindir}/kibitz
%{bindir}/tknewsbiff
%{bindir}/autopasswd
%{bindir}/autoexpect
%{bindir}/tkpasswd
%{bindir}/dislocate
%{bindir}/lpunlock
%{app_prefix}/include/expect_comm.h
%{app_prefix}/include/expect.h
%{app_prefix}/include/expect_tcl.h
%{app_prefix}/%{lib}/libexpect%{expect_minor}.a
%{app_prefix}/%{lib}/expect%{expect_minor}/libexpect%{expect_minor}.a
%{sharedir}/doc/README.LSB.expect.build.patches

%changelog
* Wed May 23 2012 Stew Benedict <stewb@linux-foundation.org>
- Bundle the patched tcl tests in the package (P5, bug 3240)

* Wed Jun 15 2011 Stew Benedict <stewb@linux-foundation.org>
- bump to tcl/tk to 8.5.9, re-diff P2, add hunk for bug 3260

* Mon May 16 2011 Stew Benedict <stewb@linux-foundation.org>
- use spec file to build, drop nALFS, tcl/tk/expect all from 1 spec file now

